library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ram is
   generic
	(
		address_width_ram 	: integer := 8;
		data_width_ram 		: integer := 8
	);
	
	port
	(
		clock_ram				: in  std_logic;
		data_in_ram 			: in  std_logic_vector(0 to data_width_ram - 1);
		data_out_ram			: out std_logic_vector(0 to data_width_ram - 1);
		write_enable_ram  	: in  std_logic;	
		write_address_ram 	: in  std_logic_vector(0 to address_width_ram - 1);	
		read_enable_ram      : in 	std_logic;				
		read_address_ram  	: in  std_logic_vector(0 to address_width_ram - 1)
	);
end ram;

architecture ram_rtl of ram is
	type ram_type is array(0 TO 2 ** address_width_ram - 1) of std_logic_vector(0 to data_width_ram - 1);
	signal ram : ram_type;
begin

	-- This is a dual port ram
	read_write : process (clock_ram)
	begin
		if rising_edge(clock_ram)
		then	
		   data_out_ram <= (others => '0');
			if write_enable_ram = '1'
			then
			   -- Write to the RAM
				ram(to_integer(unsigned(write_address_ram))) <= data_in_ram;
			end if;
			-- Read from RAM
			if read_enable_ram = '1'
			then
				data_out_ram <= ram(to_integer(unsigned(read_address_ram)));
			end if;
		end if;
	end process;
end architecture ram_rtl;